@@ -20,10 +20,13 @@
 #ifndef __GSL_LINALG_H__
 #define __GSL_LINALG_H__
 
+#include <stdlib.h>
 #include <gsl/gsl_mode.h>
 #include <gsl/gsl_permutation.h>
 #include <gsl/gsl_vector.h>
 #include <gsl/gsl_matrix.h>
+#include <gsl/gsl_math.h>
+#include <gsl/gsl_inline.h>
 
 #undef __BEGIN_DECLS
 #undef __END_DECLS
@@ -167,6 +170,8 @@ gsl_linalg_SV_solve (const gsl_matrix * U,
                      const gsl_vector * b,
                      gsl_vector * x);
 
+int gsl_linalg_SV_leverage(const gsl_matrix *U, gsl_vector *h);
+
 
 /* LU Decomposition, Gaussian elimination with partial pivoting
  */
@@ -450,6 +455,9 @@ int gsl_linalg_complex_cholesky_solve (const gsl_matrix_complex * cholesky,
 int gsl_linalg_complex_cholesky_svx (const gsl_matrix_complex * cholesky,
                                      gsl_vector_complex * x);
 
+int gsl_linalg_complex_cholesky_invert(gsl_matrix_complex * cholesky);
+
+
 /* Symmetric to symmetric tridiagonal decomposition */
 
 int gsl_linalg_symmtd_decomp (gsl_matrix * A, 
@@ -583,6 +591,55 @@ int gsl_linalg_balance_matrix (gsl_matrix * A, gsl_vector * D);
 int gsl_linalg_balance_accum (gsl_matrix * A, gsl_vector * D);
 int gsl_linalg_balance_columns (gsl_matrix * A, gsl_vector * D);
 
+INLINE_DECL void gsl_linalg_givens (const double a, const double b,
+                                    double *c, double *s);
+INLINE_DECL void gsl_linalg_givens_gv (gsl_vector * v, const size_t i,
+                                       const size_t j, const double c,
+                                       const double s);
+
+#ifdef HAVE_INLINE
+
+/* Generate a Givens rotation (cos,sin) which takes v=(x,y) to (|v|,0) 
+   From Golub and Van Loan, "Matrix Computations", Section 5.1.8 */
+INLINE_FUN
+void
+gsl_linalg_givens (const double a, const double b, double *c, double *s)
+{
+  if (b == 0)
+    {
+      *c = 1;
+      *s = 0;
+    }
+  else if (fabs (b) > fabs (a))
+    {
+      double t = -a / b;
+      double s1 = 1.0 / sqrt (1 + t * t);
+      *s = s1;
+      *c = s1 * t;
+    }
+  else
+    {
+      double t = -b / a;
+      double c1 = 1.0 / sqrt (1 + t * t);
+      *c = c1;
+      *s = c1 * t;
+    }
+} /* gsl_linalg_givens() */
+
+INLINE_FUN
+void
+gsl_linalg_givens_gv (gsl_vector * v, const size_t i, const size_t j,
+                      const double c, const double s)
+{
+  /* Apply rotation to vector v' = G^T v */
+
+  double vi = gsl_vector_get (v, i);
+  double vj = gsl_vector_get (v, j);
+  gsl_vector_set (v, i, c * vi - s * vj);
+  gsl_vector_set (v, j, s * vi + c * vj);
+}
+
+#endif /* HAVE_INLINE */
 
 __END_DECLS
 
