@@ -1,10 +1,10 @@
 /* linalg/qr.c
  * 
- * Copyright (C) 1996, 1997, 1998, 1999, 2000 Gerard Jungman, Brian Gough
+ * Copyright (C) 1996, 1997, 1998, 1999, 2000, 2007 Gerard Jungman, Brian Gough
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or (at
+ * the Free Software Foundation; either version 3 of the License, or (at
  * your option) any later version.
  * 
  * This program is distributed in the hope that it will be useful, but
@@ -14,7 +14,7 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 /* Author:  G. Jungman */
@@ -22,16 +22,12 @@
 #include <config.h>
 #include <stdlib.h>
 #include <string.h>
+#include <gsl/gsl_linalg.h>
 #include <gsl/gsl_math.h>
 #include <gsl/gsl_vector.h>
 #include <gsl/gsl_matrix.h>
 #include <gsl/gsl_blas.h>
 
-#include <gsl/gsl_linalg.h>
-
-#define REAL double
-
-#include "givens.c"
 #include "apply_givens.c"
 
 /* Factorise a general M x N matrix A into
@@ -344,7 +340,7 @@ gsl_linalg_QR_QTvec (const gsl_matrix * QR, const gsl_vector * tau, gsl_vector *
     }
   else if (v->size != M)
     {
-      GSL_ERROR ("vector size must be N", GSL_EBADLEN);
+      GSL_ERROR ("vector size must be M", GSL_EBADLEN);
     }
   else
     {
@@ -377,7 +373,7 @@ gsl_linalg_QR_Qvec (const gsl_matrix * QR, const gsl_vector * tau, gsl_vector *
     }
   else if (v->size != M)
     {
-      GSL_ERROR ("vector size must be N", GSL_EBADLEN);
+      GSL_ERROR ("vector size must be M", GSL_EBADLEN);
     }
   else
     {
@@ -385,7 +381,7 @@ gsl_linalg_QR_Qvec (const gsl_matrix * QR, const gsl_vector * tau, gsl_vector *
 
       /* compute Q^T v */
 
-      for (i = GSL_MIN (M, N); i > 0 && i--;)
+      for (i = GSL_MIN (M, N); i-- > 0;)
         {
           gsl_vector_const_view c = gsl_matrix_const_column (QR, i);
           gsl_vector_const_view h = gsl_vector_const_subvector (&(c.vector), 
@@ -398,6 +394,39 @@ gsl_linalg_QR_Qvec (const gsl_matrix * QR, const gsl_vector * tau, gsl_vector *
     }
 }
 
+/* Form the product Q^T A  from a QR factorized matrix */
+
+int
+gsl_linalg_QR_QTmat (const gsl_matrix * QR, const gsl_vector * tau, gsl_matrix * A)
+{
+  const size_t M = QR->size1;
+  const size_t N = QR->size2;
+
+  if (tau->size != GSL_MIN (M, N))
+    {
+      GSL_ERROR ("size of tau must be MIN(M,N)", GSL_EBADLEN);
+    }
+  else if (A->size1 != M)
+    {
+      GSL_ERROR ("matrix must have M rows", GSL_EBADLEN);
+    }
+  else
+    {
+      size_t i;
+
+      /* compute Q^T A */
+
+      for (i = 0; i < GSL_MIN (M, N); i++)
+        {
+          gsl_vector_const_view c = gsl_matrix_const_column (QR, i);
+          gsl_vector_const_view h = gsl_vector_const_subvector (&(c.vector), i, M - i);
+          gsl_matrix_view m = gsl_matrix_submatrix(A, i, 0, M - i, A->size2);
+          double ti = gsl_vector_get (tau, i);
+          gsl_linalg_householder_hm (ti, &(h.vector), &(m.matrix));
+        }
+      return GSL_SUCCESS;
+    }
+}
 
 /*  Form the orthogonal matrix Q from the packed QR matrix */
 
@@ -427,7 +456,7 @@ gsl_linalg_QR_unpack (const gsl_matrix * QR, const gsl_vector * tau, gsl_matrix
 
       gsl_matrix_set_identity (Q);
 
-      for (i = GSL_MIN (M, N); i > 0 && i--;)
+      for (i = GSL_MIN (M, N); i-- > 0;)
         {
           gsl_vector_const_view c = gsl_matrix_const_column (QR, i);
           gsl_vector_const_view h = gsl_vector_const_subvector (&c.vector,
@@ -496,14 +525,14 @@ gsl_linalg_QR_update (gsl_matrix * Q, gsl_matrix * R,
          simultaneously applied to R,  H = J_1^T ... J^T_(n-1) R
          so that H is upper Hessenberg.  (12.5.2) */
 
-      for (k = M - 1; k > 0; k--)
+      for (k = M - 1; k > 0; k--)  /* loop from k = M-1 to 1 */
         {
           double c, s;
           double wk = gsl_vector_get (w, k);
           double wkm1 = gsl_vector_get (w, k - 1);
 
-          create_givens (wkm1, wk, &c, &s);
-          apply_givens_vec (w, k - 1, k, c, s);
+          gsl_linalg_givens (wkm1, wk, &c, &s);
+          gsl_linalg_givens_gv (w, k - 1, k, c, s);
           apply_givens_qr (M, N, Q, R, k - 1, k, c, s);
         }
 
@@ -527,7 +556,7 @@ gsl_linalg_QR_update (gsl_matrix * Q, gsl_matrix * R,
           double diag = gsl_matrix_get (R, k - 1, k - 1);
           double offdiag = gsl_matrix_get (R, k, k - 1);
 
-          create_givens (diag, offdiag, &c, &s);
+          gsl_linalg_givens (diag, offdiag, &c, &s);
           apply_givens_qr (M, N, Q, R, k - 1, k, c, s);
 
           gsl_matrix_set (R, k, k - 1, 0.0);    /* exact zero of G^T */
