@@ -249,8 +249,16 @@ FUNCTION (test, text) (const size_t M, const size_t N)
   size_t i, j;
   int k = 0;
 
+  char filename[] = "test.XXXXXX";
+#if !defined( _MSC_VER )
+  int fd = mkstemp(filename);
+#else
+  char * fd = _mktemp(filename);
+# define fdopen fopen
+#endif
+
   {
-    FILE *f = fopen ("test.txt", "w");
+    FILE *f = fdopen (fd, "w");
     k = 0;
     for (i = 0; i < M; i++)
       {
@@ -270,7 +278,7 @@ FUNCTION (test, text) (const size_t M, const size_t N)
   }
 
   {
-    FILE *f = fopen ("test.txt", "r");
+    FILE *f = fopen (filename, "r");
     TYPE (gsl_matrix) * mm = FUNCTION (gsl_matrix, alloc) (M, N);
     status = 0;
 
@@ -293,6 +301,7 @@ FUNCTION (test, text) (const size_t M, const size_t N)
     FUNCTION (gsl_matrix, free) (mm);
   }
 
+  unlink(filename);
   FUNCTION (gsl_matrix, free) (m);
 }
 #endif
@@ -305,8 +314,16 @@ FUNCTION (test, binary) (const size_t M, const size_t N)
   size_t i, j;
   int k = 0;
 
+  char filename[] = "test.XXXXXX";
+#if !defined( _MSC_VER )
+  int fd = mkstemp(filename);
+#else
+  char * fd = _mktemp(filename);
+# define fdopen fopen
+#endif
+
   {
-    FILE *f = fopen ("test.dat", "wb");
+    FILE *f = fdopen (fd, "wb");
     k = 0;
     for (i = 0; i < M; i++)
       {
@@ -326,7 +343,7 @@ FUNCTION (test, binary) (const size_t M, const size_t N)
   }
 
   {
-    FILE *f = fopen ("test.dat", "rb");
+    FILE *f = fopen (filename, "rb");
     TYPE (gsl_matrix) * mm = FUNCTION (gsl_matrix, alloc) (M, N);
     status = 0;
 
@@ -349,20 +366,30 @@ FUNCTION (test, binary) (const size_t M, const size_t N)
     FUNCTION (gsl_matrix, free) (mm);
   }
 
+  unlink(filename);
+
   FUNCTION (gsl_matrix, free) (m);
 }
 
 void
 FUNCTION (test, binary_noncontiguous) (const size_t M, const size_t N)
 {
-  TYPE (gsl_matrix) * l = FUNCTION (gsl_matrix, alloc) (M+1, N+1);
+  TYPE (gsl_matrix) * l = FUNCTION (gsl_matrix, calloc) (M+1, N+1);
   VIEW (gsl_matrix, view) m = FUNCTION (gsl_matrix, submatrix) (l, 0, 0, M, N);
 
   size_t i, j;
   int k = 0;
 
+  char filename[] = "test.XXXXXX";
+#if !defined( _MSC_VER )
+  int fd = mkstemp(filename);
+#else
+  char * fd = _mktemp(filename);
+# define fdopen fopen
+#endif
+
   {
-    FILE *f = fopen ("test.dat", "wb");
+    FILE *f = fdopen (fd, "wb");
     k = 0;
     for (i = 0; i < M; i++)
       {
@@ -382,7 +409,7 @@ FUNCTION (test, binary_noncontiguous) (const size_t M, const size_t N)
   }
 
   {
-    FILE *f = fopen ("test.dat", "rb");
+    FILE *f = fopen (filename, "rb");
     TYPE (gsl_matrix) * ll = FUNCTION (gsl_matrix, alloc) (M+1, N+1);
     VIEW (gsl_matrix, view) mm = FUNCTION (gsl_matrix, submatrix) (ll, 0, 0, M, N);
     status = 0;
@@ -406,6 +433,7 @@ FUNCTION (test, binary_noncontiguous) (const size_t M, const size_t N)
     FUNCTION (gsl_matrix, free) (ll);
   }
 
+  unlink(filename);
   FUNCTION (gsl_matrix, free) (l);
 }
 
@@ -577,7 +605,7 @@ FUNCTION (test, ops) (const size_t P, const size_t Q)
             if (GSL_REAL (z) != (ATOMIC) (-5)
                 || GSL_IMAG (z) != (ATOMIC) (-10))
               status = 1;
-            k++;
+            k++; 
           }
       }
     gsl_test (status, NAME (gsl_matrix) "_sub matrix subtraction");
