@@ -22,6 +22,7 @@ void FUNCTION (test, ops) (const size_t M, const size_t N);
 void FUNCTION (test, trap) (const size_t M, const size_t N);
 void FUNCTION (test, text) (const size_t M, const size_t N);
 void FUNCTION (test, binary) (const size_t M, const size_t N);
+void FUNCTION (test, binary_noncontiguous) (const size_t M, const size_t N);
 
 #define TEST(expr,desc) gsl_test((expr), NAME(gsl_matrix) desc " M=%d, N=%d", M, N)
 
@@ -417,7 +418,6 @@ void
 FUNCTION (test, ops) (const size_t M, const size_t N)
 {
   size_t i, j;
-  size_t k = 0;
   
   TYPE (gsl_matrix) * a = FUNCTION (gsl_matrix, calloc) (M, N);
   TYPE (gsl_matrix) * b = FUNCTION (gsl_matrix, calloc) (M, N);
@@ -580,7 +580,7 @@ FUNCTION (test, ops) (const size_t M, const size_t N)
           {
             BASE r = FUNCTION(gsl_matrix,get) (m,i,j);
             BASE x = FUNCTION(gsl_matrix,get) (a,i,j);
-            BASE y = (i == j) ? (x + 5.0) : x;
+            BASE y = (i == j) ? (x + (ATOMIC) 5.0) : x;
             if (fabs(r - y) > 2 * GSL_FLT_EPSILON * fabs(y))
               status = 1;
           }
@@ -622,8 +622,16 @@ FUNCTION (test, text) (const size_t M, const size_t N)
   size_t i, j;
   int k = 0;
 
+  char filename[] = "test.XXXXXX";
+#if !defined( _MSC_VER )
+  int fd = mkstemp(filename);
+#else
+  char * fd = _mktemp(filename);
+# define fdopen fopen
+#endif
+
   {
-    FILE *f = fopen ("test.txt", "w");
+    FILE *f = fdopen(fd, "w");
 
     for (i = 0; i < M; i++)
       {
@@ -639,7 +647,7 @@ FUNCTION (test, text) (const size_t M, const size_t N)
   }
 
   {
-    FILE *f = fopen ("test.txt", "r");
+    FILE *f = fopen (filename, "r");
     TYPE (gsl_matrix) * mm = FUNCTION (gsl_matrix, alloc) (M, N);
     status = 0;
 
@@ -661,6 +669,8 @@ FUNCTION (test, text) (const size_t M, const size_t N)
     FUNCTION (gsl_matrix, free) (mm);
   }
 
+  unlink(filename);
+
   FUNCTION (gsl_matrix, free) (m);
 }
 #endif
@@ -673,8 +683,16 @@ FUNCTION (test, binary) (const size_t M, const size_t N)
   size_t i, j;
   size_t k = 0;
 
+  char filename[] = "test.XXXXXX";
+#if !defined( _MSC_VER )
+  int fd = mkstemp(filename);
+#else
+  char * fd = _mktemp(filename);
+# define fdopen fopen
+#endif
+
   {
-    FILE *f = fopen ("test.dat", "wb");
+    FILE *f = fdopen(fd, "wb");
     k = 0;
     for (i = 0; i < M; i++)
       {
@@ -690,7 +708,7 @@ FUNCTION (test, binary) (const size_t M, const size_t N)
   }
 
   {
-    FILE *f = fopen ("test.dat", "rb");
+    FILE *f = fopen (filename, "rb");
     TYPE (gsl_matrix) * mm = FUNCTION (gsl_matrix, alloc) (M, N);
     status = 0;
 
@@ -712,9 +730,73 @@ FUNCTION (test, binary) (const size_t M, const size_t N)
     FUNCTION (gsl_matrix, free) (mm);
   }
 
+  unlink(filename);
+
   FUNCTION (gsl_matrix, free) (m);
 }
 
+void
+FUNCTION (test, binary_noncontiguous) (const size_t M, const size_t N)
+{
+  TYPE (gsl_matrix) * l = FUNCTION (gsl_matrix, calloc) (M+1, N+1);
+  VIEW (gsl_matrix, view) m = FUNCTION (gsl_matrix, submatrix) (l, 0, 0, M, N);
+
+  size_t i, j;
+  size_t k = 0;
+
+  char filename[] = "test.XXXXXX";
+#if !defined( _MSC_VER )
+  int fd = mkstemp(filename);
+#else
+  char * fd = _mktemp(filename);
+# define fdopen fopen
+#endif
+
+  {
+    FILE *f = fdopen(fd, "wb");
+    k = 0;
+    for (i = 0; i < M; i++)
+      {
+        for (j = 0; j < N; j++)
+          {
+            k++;
+            FUNCTION (gsl_matrix, set) (&m.matrix, i, j, (BASE) k);
+          }
+      }
+
+    FUNCTION (gsl_matrix, fwrite) (f, &m.matrix);
+    fclose (f);
+  }
+
+  {
+    FILE *f = fopen (filename, "rb");
+    TYPE (gsl_matrix) * ll = FUNCTION (gsl_matrix, alloc) (M+1, N+1);
+    VIEW (gsl_matrix, view) mm = FUNCTION (gsl_matrix, submatrix) (ll, 0, 0, M, N);
+    status = 0;
+
+    FUNCTION (gsl_matrix, fread) (f, &mm.matrix);
+    k = 0;
+    for (i = 0; i < M; i++)
+      {
+        for (j = 0; j < N; j++)
+          {
+            k++;
+            if (FUNCTION (gsl_matrix, get) (&mm.matrix, i, j) != (BASE) k)
+              status = 1;
+          }
+      }
+
+    gsl_test (status, NAME (gsl_matrix) "_write and read (noncontiguous)");
+
+    fclose (f);
+    FUNCTION (gsl_matrix, free) (ll);
+  }
+
+  unlink(filename);
+
+  FUNCTION (gsl_matrix, free) (l);
+}
+
 void
 FUNCTION (test, trap) (const size_t M, const size_t N)
 {
