@@ -41,23 +41,27 @@ __BEGIN_DECLS
 
 typedef struct 
 {
-  size_t n; /* number of observations */
-  size_t p; /* number of parameters */
-  gsl_matrix * A;
+  size_t nmax;         /* maximum number of observations */
+  size_t pmax;         /* maximum number of parameters */
+  size_t n;            /* number of observations in current SVD decomposition */
+  size_t p;            /* number of parameters in current SVD decomposition */
+  gsl_matrix * A;      /* least squares matrix for SVD, n-by-p */
   gsl_matrix * Q;
   gsl_matrix * QSI;
   gsl_vector * S;
   gsl_vector * t;
   gsl_vector * xt;
   gsl_vector * D;
+
+  gsl_matrix * Linv;   /* pseudo inverse of regularization matrix, p-by-p */
 } 
 gsl_multifit_linear_workspace;
 
 gsl_multifit_linear_workspace *
-gsl_multifit_linear_alloc (size_t n, size_t p);
+gsl_multifit_linear_alloc (const size_t n, const size_t p);
 
 void
-gsl_multifit_linear_free (gsl_multifit_linear_workspace * work);
+gsl_multifit_linear_free (gsl_multifit_linear_workspace * w);
 
 int
 gsl_multifit_linear (const gsl_matrix * X,
@@ -69,24 +73,99 @@ gsl_multifit_linear (const gsl_matrix * X,
 
 int
 gsl_multifit_linear_svd (const gsl_matrix * X,
-                         const gsl_vector * y,
-                         double tol,
-                         size_t * rank,
-                         gsl_vector * c,
-                         gsl_matrix * cov,
-                         double *chisq, 
                          gsl_multifit_linear_workspace * work);
 
 int
-gsl_multifit_linear_usvd (const gsl_matrix * X,
-                          const gsl_vector * y,
-                          double tol,
-                          size_t * rank,
-                          gsl_vector * c,
-                          gsl_matrix * cov,
-                          double *chisq, 
+gsl_multifit_linear_bsvd (const gsl_matrix * X,
                           gsl_multifit_linear_workspace * work);
 
+int
+gsl_multifit_linear_solve (const double lambda,
+                           const gsl_matrix * X,
+                           const gsl_vector * y,
+                           gsl_vector * c,
+                           double *rnorm,
+                           double *snorm,
+                           gsl_multifit_linear_workspace * work);
+
+int
+gsl_multifit_linear_stdform1 (const gsl_vector * L,
+                              const gsl_matrix * X,
+                              const gsl_vector * y,
+                              gsl_matrix * Xs,
+                              gsl_vector * ys,
+                              gsl_multifit_linear_workspace * work);
+
+int
+gsl_multifit_linear_wstdform1 (const gsl_vector * L,
+                               const gsl_matrix * X,
+                               const gsl_vector * w,
+                               const gsl_vector * y,
+                               gsl_matrix * Xs,
+                               gsl_vector * ys,
+                               gsl_multifit_linear_workspace * work);
+
+int
+gsl_multifit_linear_stdform2 (const gsl_matrix * L,
+                              const gsl_matrix * X,
+                              const gsl_vector * y,
+                              gsl_matrix * Xs,
+                              gsl_vector * ys,
+                              gsl_matrix * M,
+                              gsl_multifit_linear_workspace * work);
+
+int
+gsl_multifit_linear_wstdform2 (const gsl_matrix * L,
+                               const gsl_matrix * X,
+                               const gsl_vector * w,
+                               const gsl_vector * y,
+                               gsl_matrix * Xs,
+                               gsl_vector * ys,
+                               gsl_matrix * M,
+                               gsl_multifit_linear_workspace * work);
+
+int
+gsl_multifit_linear_genform1 (const gsl_vector * L,
+                              gsl_vector * c,
+                              gsl_multifit_linear_workspace * work);
+
+int
+gsl_multifit_linear_genform2 (const gsl_matrix * L,
+                              const gsl_matrix * X,
+                              const gsl_vector * y,
+                              const gsl_vector * cs,
+                              const gsl_matrix * M,
+                              gsl_vector * c,
+                              gsl_multifit_linear_workspace * work);
+
+int
+gsl_multifit_linear_lreg (const double smin, const double smax,
+                          gsl_vector * reg_param);
+
+int
+gsl_multifit_linear_lcurve (const gsl_vector * y,
+                            gsl_vector * reg_param,
+                            gsl_vector * rho, gsl_vector * eta,
+                            gsl_multifit_linear_workspace * work);
+
+int
+gsl_multifit_linear_lcorner(const gsl_vector *rho,
+                            const gsl_vector *eta,
+                            size_t *idx);
+
+int
+gsl_multifit_linear_lcorner2(const gsl_vector *reg_param,
+                             const gsl_vector *eta,
+                             size_t *idx);
+
+int
+gsl_multifit_linear_Lk(const size_t p, const size_t k, gsl_matrix *L);
+
+int
+gsl_multifit_linear_Lsobolev(const size_t p, const size_t kmax,
+                             const gsl_vector *alpha, gsl_matrix *L,
+                             gsl_multifit_linear_workspace *work);
+
 int
 gsl_multifit_wlinear (const gsl_matrix * X,
                       const gsl_vector * w,
@@ -190,14 +269,23 @@ GSL_VAR const gsl_multifit_robust_type * gsl_multifit_robust_welsch;
 gsl_multifit_robust_workspace *gsl_multifit_robust_alloc(const gsl_multifit_robust_type *T,
                                                          const size_t n, const size_t p);
 void gsl_multifit_robust_free(gsl_multifit_robust_workspace *w);
-int gsl_multifit_robust_tune(const double tune, gsl_multifit_robust_workspace *w);
+int gsl_multifit_robust_tune(const double tune,
+                             gsl_multifit_robust_workspace *w);
+int gsl_multifit_robust_maxiter(const size_t maxiter,
+                                gsl_multifit_robust_workspace *w);
 const char *gsl_multifit_robust_name(const gsl_multifit_robust_workspace *w);
 gsl_multifit_robust_stats gsl_multifit_robust_statistics(const gsl_multifit_robust_workspace *w);
+int gsl_multifit_robust_weights(const gsl_vector *r, gsl_vector *wts,
+                                gsl_multifit_robust_workspace *w);
 int gsl_multifit_robust(const gsl_matrix * X, const gsl_vector * y,
                         gsl_vector * c, gsl_matrix *cov,
                         gsl_multifit_robust_workspace *w);
 int gsl_multifit_robust_est(const gsl_vector * x, const gsl_vector * c,
                             const gsl_matrix * cov, double *y, double *y_err);
+int gsl_multifit_robust_residuals(const gsl_matrix * X,
+                                  const gsl_vector * y,
+                                  const gsl_vector * c, gsl_vector * r,
+                                  gsl_multifit_robust_workspace * w);
 
 __END_DECLS
 
