@@ -524,7 +524,7 @@ hyperg_U_infinite_sum_simple(int N, double a, double bint, double b, double beps
         t_err = a0_err + b0_err;
         dchu_val += t_val;
         dchu_err += t_err;
-        if(!finite(t_val) || fabs(t_val) < EPS*fabs(dchu_val)) break;
+        if(!gsl_finite(t_val) || fabs(t_val) < EPS*fabs(dchu_val)) break;
       }
 
       result->val  = dchu_val;
@@ -1714,7 +1714,7 @@ gsl_sf_hyperg_U_e10_e(const double a, const double b, const double x,
 int
 gsl_sf_hyperg_U_int_e(const int a, const int b, const double x, gsl_sf_result * result)
 {
-  gsl_sf_result_e10 re;
+  gsl_sf_result_e10 re = {0};
   int stat_U = gsl_sf_hyperg_U_int_e10_e(a, b, x, &re);
   int stat_c = gsl_sf_result_smash_e(&re, result);
   return GSL_ERROR_SELECT_2(stat_c, stat_U);
@@ -1724,7 +1724,7 @@ gsl_sf_hyperg_U_int_e(const int a, const int b, const double x, gsl_sf_result *
 int
 gsl_sf_hyperg_U_e(const double a, const double b, const double x, gsl_sf_result * result)
 {
-  gsl_sf_result_e10 re;
+  gsl_sf_result_e10 re = {0};
   int stat_U = gsl_sf_hyperg_U_e10_e(a, b, x, &re);
   int stat_c = gsl_sf_result_smash_e(&re, result);
   return GSL_ERROR_SELECT_2(stat_c, stat_U);
