@@ -1,6 +1,6 @@
 /* vector/test_complex_source.c
  * 
- * Copyright (C) 1996, 1997, 1998, 1999, 2000, 2007 Gerard Jungman, Brian Gough
+ * Copyright (C) 1996, 1997, 1998, 1999, 2000, 2007, 2010 Gerard Jungman, Brian Gough
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -501,7 +501,18 @@ FUNCTION (test, ops) (size_t stride1, size_t stride2, size_t N)
       FUNCTION (gsl_vector, set) (b, i, z1);
     }
   
+  {
+    int status = (FUNCTION(gsl_vector,equal) (a,b) != 0);
+    TEST2 (status, "_equal vectors unequal");
+  }
+
   FUNCTION(gsl_vector, memcpy) (v, a);
+
+  {
+    int status = (FUNCTION(gsl_vector,equal) (a,v) != 1);
+    TEST2 (status, "_equal vectors equal");
+  }
+
   FUNCTION(gsl_vector, add) (v, b);
   
   {
@@ -627,8 +638,16 @@ FUNCTION (test, file) (size_t stride, size_t N)
 
   size_t i;
 
+  char filename[] = "test.XXXXXX";
+#if !defined( _MSC_VER )
+  int fd = mkstemp(filename);
+#else
+  char * fd = _mktemp(filename);
+# define fdopen fopen
+#endif
+
   {
-    FILE *f = fopen ("test.dat", "wb");
+    FILE *f = fdopen(fd, "wb");
 
     for (i = 0; i < N; i++)
       {
@@ -644,7 +663,7 @@ FUNCTION (test, file) (size_t stride, size_t N)
   }
 
   {
-    FILE *f = fopen ("test.dat", "rb");
+    FILE *f = fopen(filename, "rb");
 
     FUNCTION (gsl_vector, fread) (f, w);
 
@@ -657,11 +676,12 @@ FUNCTION (test, file) (size_t stride, size_t N)
     fclose (f);
   }
 
+  unlink(filename);
+
   FUNCTION (gsl_vector, free) (v);
   FUNCTION (gsl_vector, free) (w);
 
   gsl_test (status, NAME (gsl_vector) "_write and read work");
-
 }
 
 #if USES_LONGDOUBLE && ! HAVE_PRINTF_LONGDOUBLE
@@ -675,8 +695,16 @@ FUNCTION (test, text) (size_t stride, size_t N)
 
   size_t i;
 
+  char filename[] = "test.XXXXXX";
+#if !defined( _MSC_VER )
+  int fd = mkstemp(filename);
+#else
+  char * fd = _mktemp(filename);
+# define fdopen fopen
+#endif
+
   {
-    FILE *f = fopen ("test.txt", "w");
+    FILE *f = fdopen(fd, "w");
 
     for (i = 0; i < N; i++)
       {
@@ -692,7 +720,7 @@ FUNCTION (test, text) (size_t stride, size_t N)
   }
 
   {
-    FILE *f = fopen ("test.txt", "r");
+    FILE *f = fopen(filename, "r");
 
     FUNCTION (gsl_vector, fscanf) (f, w);
 
@@ -705,6 +733,8 @@ FUNCTION (test, text) (size_t stride, size_t N)
     fclose (f);
   }
 
+  unlink(filename);
+
   FUNCTION (gsl_vector, free) (v);
   FUNCTION (gsl_vector, free) (w);
 
