@@ -357,7 +357,7 @@ FUNCTION (test, func) (size_t stride, size_t N)
 
     for (i = 0; i < N; i++)
       {
-        if (FUNCTION (gsl_vector, get) (v, i) != (ATOMIC) (i*2.0))
+        if (FUNCTION (gsl_vector, get) (v, i) != (ATOMIC) ((ATOMIC)i*(ATOMIC)2.0))
           status = 1;
       };
 
@@ -371,7 +371,7 @@ FUNCTION (test, func) (size_t stride, size_t N)
 
     for (i = 0; i < N; i++)
       {
-        if (FUNCTION (gsl_vector, get) (v, i) != (ATOMIC) (i*2.0 + 7))
+        if (FUNCTION (gsl_vector, get) (v, i) != (ATOMIC) ((ATOMIC)i*(ATOMIC)2.0 + (ATOMIC)7))
           status = 1;
       };
 
@@ -718,8 +718,15 @@ FUNCTION (test, file) (size_t stride, size_t N)
 
   size_t i;
 
+  char filename[] = "test.XXXXXX";
+#if !defined( _MSC_VER )
+  int fd = mkstemp(filename);
+#else
+  char * fd = _mktemp(filename);
+#  define fdopen fopen
+#endif
   {
-    FILE *f = fopen ("test.dat", "wb");
+    FILE *f = fdopen (fd, "wb");
 
     for (i = 0; i < N; i++)
       {
@@ -732,7 +739,7 @@ FUNCTION (test, file) (size_t stride, size_t N)
   }
 
   {
-    FILE *f = fopen ("test.dat", "rb");
+    FILE *f = fopen (filename, "rb");
 
     FUNCTION (gsl_vector, fread) (f, w);
 
@@ -748,6 +755,8 @@ FUNCTION (test, file) (size_t stride, size_t N)
     fclose (f);
   }
 
+  unlink(filename);
+
   FUNCTION (gsl_vector, free) (v);      /* free whatever is in v */
   FUNCTION (gsl_vector, free) (w);      /* free whatever is in w */
 }
@@ -763,8 +772,15 @@ FUNCTION (test, text) (size_t stride, size_t N)
 
   size_t i;
 
+  char filename[] = "test.XXXXXX";
+#if !defined( _MSC_VER )
+   int fd = mkstemp(filename);
+#else
+   char * fd = _mktemp(filename);
+#  define fdopen fopen
+#endif
   {
-    FILE *f = fopen ("test.txt", "w");
+    FILE *f = fdopen (fd, "w");
 
     for (i = 0; i < N; i++)
       {
@@ -777,7 +793,7 @@ FUNCTION (test, text) (size_t stride, size_t N)
   }
 
   {
-    FILE *f = fopen ("test.txt", "r");
+    FILE *f = fopen (filename, "r");
 
     FUNCTION (gsl_vector, fscanf) (f, w);
 
@@ -793,6 +809,8 @@ FUNCTION (test, text) (size_t stride, size_t N)
     fclose (f);
   }
 
+  unlink(filename);
+
   FUNCTION (gsl_vector, free) (v);
   FUNCTION (gsl_vector, free) (w);
 }
